import 'package:dio/dio.dart';

class HTTPManager {
  Dio _dio = Dio();

  // String baseURL;
  List<CancelToken> cancelTokenList = <CancelToken>[];
  static HTTPManager? _instance;

  HTTPManager._internal();

  factory HTTPManager.getInstance() => _getInstance();

  static _getInstance() {
    // ignore: unnecessary_null_comparison
    if (_instance == null) {
      _instance = HTTPManager._internal();
    }
    return _instance;
  }

  Dio get dio => _dio;

  void init(BaseOptions options, {List<Interceptor>? interceptors}) {
    _dio.options = options;
    if (interceptors != null) {
      _dio.interceptors.addAll(interceptors);
    }
  }

  void addCancelToken(CancelToken cancelToken) {
    cancelTokenList.add(cancelToken);
  }

  void removeCancelToken(CancelToken cancelToken) {
    cancelTokenList.remove(cancelToken);
  }

  void removeAllCancelToken() {
    cancelTokenList.forEach((element) {
      element.cancel();
    });
    cancelTokenList.clear();
  }

  Future<Response<T>> get<T>(String path, {Map<String, dynamic>? queryParameters, Options? options, CancelToken? cancelToken, ProgressCallback? onReceiveProgress}) {
    cancelToken = cancelToken ?? CancelToken();
    addCancelToken(cancelToken);
    var future = _dio.get<T>(path, queryParameters: queryParameters, options: options, cancelToken: cancelToken, onReceiveProgress: onReceiveProgress);
    removeCancelToken(cancelToken);
    return future;
  }

  Future<Response<T>> post<T>(String path, {data, Map<String, dynamic>? queryParameters, Options? options, CancelToken? cancelToken, ProgressCallback? onSendProgress, ProgressCallback? onReceiveProgress}) {
    cancelToken = cancelToken ?? CancelToken();
    addCancelToken(cancelToken);
    Future<Response<T>> future = _dio.post(path, data: data, queryParameters: queryParameters, options: options, cancelToken: cancelToken, onSendProgress: onReceiveProgress, onReceiveProgress: onReceiveProgress);
    removeCancelToken(cancelToken);
    return future;
  }
}
